// ============================================================================
// $Id: act.intermud.cc,v 1.68 1998/06/08 15:34:29 chris Exp $
//
// This is part of Realm Of Magic
//
// Usage: Intermud interface code to CircleMUD
//
// All rights reserved. See license.doc for complete information.
//
// Copyright (C) 1995 by the Hackerz of the Realm Of Magic
// This file is taken from Cris Austin (Stryker@PageMaster) (C) 1995
// Realm Of Magic is based on CircleMUD, Copyright (C) 1993.
// CircleMUD is based on DikuMUD, Copyright (C) 1990, 1991.
// ============================================================================

#include <cstdio>
#include <cstdlib>
#include <cctype>
#include <climits>
#include <cerrno>
#include <cstring>

extern "C" {
  #include <sys/types.h>
  #include <sys/socket.h>
  #include <sys/wait.h>
  #include <sys/time.h>
  #include <sys/resource.h>
  #include <sys/un.h>
  #include <netdb.h>
  #include <fcntl.h>
  #include <signal.h>
  #include <netinet/in.h>
  #include <unistd.h>
}

#include "structs.hh"
#include "utils.hh"
#include "comm.hh"
#include "db.hh"
#include "interpreter.hh"
#include "handler.hh"
#include "descr_more.hh"

#include "structs.hh"
#include "descr_data.hh"

#include "intermud.hh"
#include "screen.hh"
#include "utility.hh"
#include "utils.hh"
#include "constants.hh"
#include "intermud_comm.hh"

#include "tokenizer.hh"

#include "base/mobile.hh"
#include "base/mobile_flags.hh"
#include "base/playerflags.hh"

#include "commands/arglist.hh"

void serv_recv_info(string);
void serv_recv_mudlistrpy(string, string);
void serv_recv_intertell(string, string, string, string);
void serv_recv_intertellrpy(string, string, string, string);
void serv_recv_interwhoreq(string, string);
void serv_recv_interwhorpy(string, string, string);
void serv_recv_interpage(string, string, string, string);
void serv_recv_mudinfo(string, string);
void serv_recv_interwiz(string, string, string);
void serv_recv_stats(string, string);

extern _intermud intermud;

ACMD(do_mudinfo)
{
  ArgumentList args(argument);
  
  if (args.argn() == 0 || args.argn() > 1) {
    *ch << "Usage:\r\nmudinfo <mudname>\r\n";
    return;
  }
  
  if (!intermud.Connection()) {
    *ch << "The Intermud Server seems to be down! Contact a God!\r\n";
    return;
  }
  
  if (PRF_FLAGGED(ch, PRF_NOINTERMUD)) {
    *ch << "You can't use Intermud when having NOINTERMUD switched on!\r\n";
    return;
  }
    
  if (!str_cmp(args[0], MUDNAME)) {
    *ch << "You are currently playing here! You can get all info you want!\r\n";
    return;
  }

  intermud << "mudinfo#@#" << args[0] << "#@#" << ch->GetString(name) << sendit;
  return;
}

ACMD(do_mudlist)
{
  ArgumentList args(argument);
  
  if (args.argn() != 0) {
    *ch << "Usage:\r\nmudlist\r\n";
    return;
  }
  
  if (!intermud.Connection()) {
    *ch << "The Intermud Server seems to be down! Contact a God!\r\n";
    return;
  }
  
  if (PRF_FLAGGED(ch, PRF_NOINTERMUD)) {
    *ch << "You can't use Intermud when having NOINTERMUD switched on!\r\n";
    return;
  }
 
  intermud << "mudlist#@#" << ch->GetString(name) << sendit;
  return;
}

ACMD(do_interpage)
{
  ArgumentList args(argument);
  
  if (args.argn() == 0) {
    *ch << "Usage:\r\nipage User@Mud <text>\r\n";
    return;
  }
  
  if (!intermud.Connection()) {
    *ch << "The Intermud Server seems to be down! Contact a God!\r\n";
    return;
  }
  
  if (PRF_FLAGGED(ch, PRF_NOINTERMUD)) {
    *ch << "You can't use Intermud when having NOINTERMUD switched on!\r\n";
    return;
  }
 
  tokenizer dest(args[0], "@");
  
  if (dest.size() != 2) {
    *ch << "Destination must be in User@Mud format!\r\n";
    return;
  }

  if (!str_cmp(dest[1], MUDNAME)) {
    *ch << "Please use 'page' for that!\r\n";
    return;
  }
   
  *ch << "\a\a*" << ch << "@" << MUDNAME << "* " 
      << ((args.argn() > 1) ? args.args(1) : " ") << "\r\n";
  intermud << "interpage#@#" << dest[0] << "#@#" << dest[1] << "#@#" 
           << ch->GetString(name) << "#@#" 
           << ((args.argn() > 1) ? args.args(1) : " ") << sendit;
  return;
}


ACMD(do_interwho)
{
  ArgumentList args(argument);
  
  if (args.argn() == 0 || args.argn() > 1) {
    *ch << "Usage:\r\niwho <mudname>\r\n";
    return;
  }
  
  if (!intermud.Connection()) {
    *ch << "The Intermud Server seems to be down! Contact a God!\r\n";
    return;
  }
  
  if (PRF_FLAGGED(ch, PRF_NOINTERMUD)) {
    *ch << "You can't use Intermud when having NOINTERMUD switched on!\r\n";
    return;
  }
 
  if (!str_cmp(args[0], MUDNAME)) {
    *ch << "Please use 'who' for that!\r\n";
    return;
  }
    
  intermud << "who-query#@#" << args[0] << "#@#" << ch->GetString(name) 
           << sendit;
  *ch << "Request sent, but it might take a minute.\r\n";
  return;
}

ACMD(do_interreply)
{
  ArgumentList args(argument);
  
  if (!intermud.Connection()) {
    *ch << "The Intermud Server seems to be down! Contact a God!\r\n";
    return;
  }
  
  if (PRF_FLAGGED(ch, PRF_NOINTERMUD)) {
    *ch << "You can't use Intermud when having NOINTERMUD switched on!\r\n";
    return;
  }
  
  if (!ch->GetString(IM_reply)) {
    *ch << "No one told you anything!\r\n";
    return;
  }

  if (args.argn() == 0) {
    *ch << "What do you want to interreply?\r\n";
    return;
  }

  intermud << "intertell#@#" << ch->GetString(IM_reply) << "#@#" 
           << ch->GetString(IM_mud) << "#@#" << ch->GetString(name) << "#@#" 
           << args.args() << sendit;
  *ch << "%C1You reply to " << capnext << ch->GetString(IM_reply) << "@"
      << capnext << ch->GetString(IM_mud) << ", '" << args.args() 
      << "'%C0\r\n";
  return;
}
  
  

ACMD(do_intertell)
{
  ArgumentList args(argument);
  
  if (args.argn() < 2) {
    *ch << "Usage:\r\nitell User@Mud <text>\r\n";
    return;
  }
  
  if (!intermud.Connection()) {
    *ch << "The Intermud Server seems to be down! Contact a God!\r\n";
    return;
  }
  
  if (PRF_FLAGGED(ch, PRF_NOINTERMUD)) {
    *ch << "You can't use Intermud when having NOINTERMUD switched on!\r\n";
    return;
  }
 
  tokenizer dest(args[0], "@");
  
  if (dest.size() != 2) {
    *ch << "Destination must be in User@Mud format!\r\n";
    return;
  }
    
  if (!str_cmp(dest[1], MUDNAME)) {
    *ch << "Please use 'tell' for that!\r\n";
    return;
  }
      
  intermud << "intertell#@#" << dest[0] << "#@#" << dest[1] << "#@#" 
           << ch->GetString(name) << "#@#" << args.args(1) << sendit;
    
  *ch << "%C1You tell " << capnext << dest[0] << "@" << capnext << dest[1]
      << ", '" << args.args(1) << "'%C0\r\n";

  return;
}

ACMD(do_interwiz)
{
  ArgumentList args(argument);
  
  if (args.argn() == 0) {
    *ch << "Usage:\r\niwiz <text>\r\n";
    return;
  }
  
  if (!intermud.Connection()) {
    *ch << "The Intermud Server seems to be down! Contact a God!\r\n";
    return;
  }
    
  if (PRF_FLAGGED(ch, PRF_NOINTERMUD)) {
    *ch << "You can't use Intermud when having NOINTERMUD switched on!\r\n";
    return;
  }
    
  struct descriptor_data *d;
   
  for (d = descriptor_list; d; d = d->next)  
    if (!d->connected && d->character && 
        (!PRF_FLAGGED(d->character, PRF_NOWIZ)) &&
        (!PLR_FLAGGED(d->character, PLR_WRITING)) &&
        !PRF_FLAGGED(d->character, PRF_NOINTERMUD) &&
        (d->character->GetLevel() >= LEVEL_IMMORT))  
       *d->character << "%C5" << ch << "@" << MUDNAME << ": "
                     << args.args() << "%C0\r\n";
      
  intermud << "interwiz#@#" << ch->GetString(name) << "#@#" << args.args() 
           << sendit;
  return; 
}

void incoming_intermud_message()
{
  string message;
 
  message = intermud.ReceiveData();
  tokenizer message_tokens(message, "#@#");
  
  if (!case_compare(message_tokens[0], "mudlog"))
    serv_recv_info(message_tokens[1]);
  else if (!case_compare(message_tokens[0], "inc-mudlist"))
    serv_recv_mudlistrpy(message_tokens[1], message_tokens[2]);
  else if (!case_compare(message_tokens[0], "inc-tell"))
    serv_recv_intertell(message_tokens[1], message_tokens[2],
                        message_tokens[3], message_tokens[4]);
  else if (!case_compare(message_tokens[0], "inc-tell-nr"))
    serv_recv_intertellrpy(message_tokens[1], message_tokens[2],
                           message_tokens[3], message_tokens[4]);
  else if (!case_compare(message_tokens[0], "inc-who-query"))
    serv_recv_interwhoreq(message_tokens[1], message_tokens[2]);
  else if (!case_compare(message_tokens[0], "inc-who-reply"))
    serv_recv_interwhorpy(message_tokens[1], message_tokens[2], 
                          message_tokens[3]);
  else if (!case_compare(message_tokens[0], "inc-interpage"))
    serv_recv_interpage(message_tokens[1], message_tokens[2],
                        message_tokens[3], message_tokens[4]);
  else if (!case_compare(message_tokens[0], "inc-mudinfo"))
    serv_recv_mudinfo(message_tokens[1], message_tokens[2]);
  else if (!case_compare(message_tokens[0], "inc-stats"))
    serv_recv_stats(message_tokens[1], message_tokens[2]);
  else if (!case_compare(message_tokens[0], "inc-interwiz"))
    serv_recv_interwiz(message_tokens[1], message_tokens[2],
                       message_tokens[3]);
  else
    logprintf("SYSERR: Message with unknown header %s received from Server.",
              message_tokens[0]);
}

void serv_recv_info(string serv_message)
{
  mudlog(serv_message.c_str(), NRM, LEVEL_GOD2, TRUE);
  return;
}

void serv_recv_mudlistrpy(string player, string data)
{
  struct descriptor_data *d;
  TDescriptor_More *dm;
  
  int idx;

  for (d = descriptor_list; d; d = d->next)
    if (!d->connected && d->character &&
        !PRF_FLAGGED(d->character, PRF_NOINTERMUD) &&
        !case_compare(player, d->character->GetString(name))) {
      string::iterator pos;
      for (pos = data.begin(); pos != data.end(); ++pos) {
        if (*pos == '\n') {
          pos = data.insert(pos, '\r');
          ++pos;
        }
      }
      dm = new TDescriptor_More(d, (char *) data.c_str());
      DM_READ(dm);
      return;
    }
  return;
}

void serv_recv_intertell(string sender, string remote_mud, string player,
                         string data)
{
  struct descriptor_data *d;
  int player_found = 0;

  for (d = descriptor_list; d; d = d->next)
    if (!d->connected && d->character &&
        !PRF_FLAGGED(d->character, PRF_NOINTERMUD) &&
        (!case_compare(player, d->character->GetString(name)))) {
      if (d->character->GetInvisLevel() >= LEVEL_IMMORT2)
        continue;
      if (PLR_FLAGGED(d->character, PLR_WRITING))
        player_found = 1;
      else {
        *d->character << "%C1" << capnext << sender << "@"
                      << capnext << remote_mud << " tells you, '"
                      << data << "'%C0\r\n";
        if (case_compare(sender, "Server")) { 
          d->character->SetString(IM_reply, sender.c_str());
          d->character->SetString(IM_mud, remote_mud.c_str());
        }
        player_found = 2;
      }
      break;
    }
  
  if (!player_found) {
    intermud << "intertell-nr#@#" << sender << "#@#"
             << remote_mud << "#@#Server#@#" << player
             << " is not on-line currently." << sendit;
  }
  else if (player_found == 1) {
    intermud << "intertell-nr#@#" << sender << "#@#"
             << remote_mud << "#@#Server#@#" << player
             << " is currently mailing or writing." << sendit;
  }
  else {
    intermud << "intertell-nr#@#" << sender << "#@#"
             << remote_mud << "#@#Server#@#"
             << "Your message has been delivered." << sendit;
  }
}

void serv_recv_intertellrpy(string sender, string remote_mud, string player,
                            string data)
{
  struct descriptor_data *d;
  int player_found = 0;

  for (d = descriptor_list; d; d = d->next)
    if (!d->connected && d->character &&
        !PRF_FLAGGED(d->character, PRF_NOINTERMUD) &&
        (!case_compare(player, d->character->GetString(name)))) {
      if (PLR_FLAGGED(d->character, PLR_WRITING))
        player_found = 1;
      else {
        *d->character << "%C1" << capnext << sender << "@"
                      << capnext << remote_mud << " tells you, '"
                      << data << "'%C0\r\n";
        if (case_compare(sender, "Server")) {
          d->character->SetString(IM_reply, sender.c_str());
          d->character->SetString(IM_mud, remote_mud.c_str());
        }
        player_found=2;
      }
      break;
    }
}

void serv_recv_interwhoreq(string sender, string remote_mud)
{
  struct descriptor_data *d;
  TMobile *ch = 0;
  int countline = 0;
  char temp[8192];
  extern char *circlemud_version;
 
  intermud << "who-reply#@#" << sender << "#@#" << remote_mud
           << "#@#";

  intermud << " ___________________________________________________________________________\n"
              "/                                                                           \\\n"
              ":                              Realm of Magic                               :\n"
              ":---------------------------------------------------------------------------:\n";
  tokenizer ver_tok(circlemud_version, "\n\r");
  intermud << ": " << ver_tok[0];
  int num_spaces = 73 - strlen(ver_tok[0]);
  for (int i = 1; i <= num_spaces; i++)
    intermud << " ";
  intermud << " :\n";
  char *upt = (char *)asctime(localtime(&boot_time));
  *(upt+strlen(upt)-1) = 0;
  long uptime = time(0)-boot_time;
  int mi=(uptime/60)%60;
  int ho=(uptime/3600)%24;
  int da=uptime/86400;
  sprintf(temp, "Up since: %s (%i day(s), %i:%02i)", upt, da, ho, mi);
  sprintf(temp, "%.73s", temp);
  intermud << ": " << temp;
  num_spaces = 73 - strlen(temp);
  for (int i =1; i <= num_spaces; i++)
    intermud << " ";
  intermud << " :\n";
  intermud << ":---------------------------------------------------------------------------:\n"
              ":               telnet rom.mud.de 4000 - http://rom.mud.de                  :\n"
              ":---------------------------------------------------------------------------:\n"
              ":                                                                           :\n";

  int player_count = 0;

  for (d = descriptor_list; d; d = d->next)
    if (!d->connected && d->character) {
      if (PRF_FLAGGED(d->character, PRF_NOINTERMUD))
        continue;
      if (d->character->GetInvisLevel() >= LEVEL_IMMORT)
        continue;
      if (d->character->GetFakeLevel() > LEVEL_IMPL)
        continue;
      if (d->character->GetFakeLevel() >= LEVEL_IMMORT) 
        sprintf(temp, "%s *%s* %s", d->character->GodTitle(),
                d->character->GetString(name), d->character->GetString(title));
      else
        sprintf(temp, "*%s* %s", 
                d->character->GetString(name),
                d->character->GetString(title));
      filter_percental_codes(temp);
      intermud << ": ";
      sprintf(temp, "%.73s", temp);
      intermud << temp;
      num_spaces = 73 - strlen(temp);
      for (int i = 1; i <= num_spaces; i++)
        intermud << " ";
      intermud << " :\n";
      player_count++; 
    }
  intermud << ":---------------------------------------------------------------------------:\n";
  sprintf(temp, "%d visible citizens of the Realm of Magic displayed.", player_count);
  sprintf(temp, "%.73s", temp);
  intermud << ": ";
  intermud << temp;
  num_spaces = 73 - strlen(temp);
  for (int i = 1; i <= num_spaces; i++)
    intermud << " ";
  intermud << " :\n";
  intermud << "\\___________________________________________________________________________/\n";
  intermud << sendit;
}

void serv_recv_interwhorpy(string player, string remote_mud, string data)
{
  TDescriptor_More *dm;
  struct descriptor_data *d;
  
  for (d = descriptor_list; d; d = d->next)
    if (!d->connected && d->character && 
        (!case_compare(player, d->character->GetString(name))) &&
        !PRF_FLAGGED(d->character, PRF_NOINTERMUD) &&
        (!PLR_FLAGGED(d->character, PLR_WRITING))) {
      string::iterator pos;
      for (pos = data.begin(); pos != data.end(); ++pos) {
        if (*pos == '\n') {
          pos = data.insert(pos, '\r');
          ++pos;
        }
      }
      dm = new TDescriptor_More(d, (char *) data.c_str());
      DM_READ(dm);

    }
}

void serv_recv_interpage(string sender, string remote_mud, string player,
                         string data)
{
  struct descriptor_data *d;
  int player_found = 0;
  
  for (d = descriptor_list; d; d = d->next)
    if (!d->connected && d->character && 
        !PRF_FLAGGED(d->character, PRF_NOINTERMUD) &&
        (!case_compare(player, d->character->GetString(name)))) {
      if (PLR_FLAGGED(d->character, PLR_WRITING))
        player_found = 1;
      else {
        *d->character << "\007\007*" << capnext << sender << "@" 
                      << capnext << remote_mud << "* " << data << "\r\n";
        player_found = 2;
      }
      break;
    }

  if (!player_found) {
    intermud << "intertell-nr#@#" << sender << "#@#"
             << remote_mud << "#@#Server#@#" << player
             << " is not on-line currently." << sendit;
  }
  if (player_found == 1) {
    intermud << "intertell-nr#@#" << sender << "#@#"
             << remote_mud << "#@#Server#@#" << player
             << " is mailing or writing currently." << sendit;
  }
}

void serv_recv_mudinfo(string player, string data)
{
  struct descriptor_data *d;
  TDescriptor_More *dm;

  for (d = descriptor_list; d; d = d->next)
    if (!d->connected && d->character && 
        !PRF_FLAGGED(d->character, PRF_NOINTERMUD) &&
        (!case_compare(player, d->character->GetString(name)))) {
      string::iterator pos;
      for (pos = data.begin(); pos != data.end(); ++pos) {
        if (*pos == '\n') {
          pos = data.insert(pos, '\r');
          ++pos;
        }
      }
      dm = new TDescriptor_More(d, (char *) data.c_str());
      DM_READ(dm);
      return;
    }

}

void serv_recv_stats(string player, string data)
{
  struct descriptor_data *d;
  TDescriptor_More *dm;

  
  for (d = descriptor_list; d; d = d->next)
    if (!d->connected && d->character && 
        !PRF_FLAGGED(d->character, PRF_NOINTERMUD) &&
        (!case_compare(player, d->character->GetString(name)))) {
        string::iterator pos;
        for (pos = data.begin(); pos != data.end(); ++pos) {
          if (*pos == '\n') {
            pos = data.insert(pos, '\r');
            ++pos;
          }
        }
        dm = new TDescriptor_More(d, (char *) data.c_str());
        DM_READ(dm);
      return;
    }
}

void serv_recv_interwiz(string sender, string remote_mud, string data)
{
  struct descriptor_data *d;

  for (d = descriptor_list; d; d = d->next)
    if (!d->connected && d->character &&
        (!PLR_FLAGGED(d->character, PLR_WRITING)) &&
        !PRF_FLAGGED(d->character, PRF_NOINTERMUD) &&
        (d->character->GetLevel() >= LEVEL_IMMORT2)) {
      *d->character << "%C5" << capnext << sender << "@" << capnext
                    << remote_mud << ": " << data << "%C0\r\n";
    }
}

ACMD(do_intermud)
{
  ArgumentList args(argument);
  
  if (args.argn() == 0) {
    *ch << "Usage:\r\n"
           "intermud <argument>\r\n"
           "possible arguments:\r\n"
           "- connect: connects the Mud to the Intermud Server\r\n"
           "- disconnect: disconnects the Mud from the Intermud Server\r\n"
           "- purge: will purge the Intermud Server's mudlists\r\n"
           "- reboot: will reboot the Intermud Server's mudlists\r\n"
           "- stats: will display the Intermud Server's network statistics\r\n"
           "- debug: will trigger the Intermud Server's debugging code\r\n"
           "- mute <mud>: will block incoming messages from the Mud <mud>\r\n"
           "- shudown: will shutdown the Intermud Server\r\n";
    return;
  }
    
  if (PRF_FLAGGED(ch, PRF_NOINTERMUD)) {
    *ch << "You can't use Intermud while having NOINTERMUD switched on!\r\n";
    return;
  }
 
  if (!str_cmp(args[0], "connect")) {
    if (intermud.Connection()) {
      *ch << "The Mud is already connected to the Intermud Server!\r\n";
    } else {
      intermud.Connect();
      mudlogprintf(MAX(LEVEL_GOD2, ch->GetInvisLevel()),
                   "(GC) Connection to Intermud by %s", ch->GetString(name));
    }
    return;
  } else if (!str_cmp(args[0], "disconnect")) {
    if (!intermud.Connection())
      *ch << "The Mud is already disconnected from the Intermud Server!\r\n";
    else {
      mudlogprintf(MAX(LEVEL_GOD2, ch->GetInvisLevel()),
                   "(GC) Intermud Server disconnect by %s", 
                   ch->GetString(name));
      intermud.CloseFD();
    }
    return;
  } if (!str_cmp(args[0], "purge")) {
    if (!intermud.Connection())
      *ch << "The Mud is not connected to the Intermud Server!\r\n";
    else
      intermud << "purge-mudlist#@#" << ch->GetString(name) << sendit;
    return;
  } else if (!str_cmp(args[0], "reboot")) {
    if (!intermud.Connection())
      *ch << "The Mud is not connected to the Intermud Server!\r\n";
    else
      intermud << "reboot-mudlist#@#" << ch->GetString(name) << sendit;
    return;
  } else if (!str_cmp(args[0], "stats")) {
    if (!intermud.Connection())
      *ch << "The Mud is not connected to the Intermud Server!\r\n";
    else
      intermud << "intermud-stats#@#" << ch->GetString(name) << sendit;
    return;
  } else if (!str_cmp(args[0], "debug")) {
    if (!intermud.Connection())
      *ch << "The Mud is not connected to the Intermud Server!\r\n";
    else
      intermud << "#@#debug#@#" << sendit;
    return;
  } else if (!str_cmp(args[0], "mute")) {
    if (!intermud.Connection())
      *ch << "The Mud is not connected to the Intermud Server!\r\n";
    else {
      if (args.argn() != 2) {
        *ch << "You must specify a remote Mud to toggle mute for!\r\n";
        return;
      }
      intermud << "mute#@#" << args[1] << "#@#" << ch->GetString(name) 
               << sendit;
    }
    return;
  } else if (!str_cmp(args[0], "shutdown")) {
    if (!intermud.Connection())
      *ch << "The Mud is not connected to the Intermud Server!\r\n";
    else {
      intermud << "shutdown#@#" << ch->GetString(name) << sendit;
      mudlogprintf(MAX(LEVEL_DEGOD, ch->GetInvisLevel()),
                   "(GC) INTERMUD: shutdown by %s", ch->GetString(name));
      intermud.CloseFD();
    }
    return;
  } else {
    *ch << "No such intermud option! Use it without argument for a list!\r\n";
    return;
  }
}


