// ============================================================================
// $Id: mud.cc,v 1.1 1996/04/22 18:25:24 questor Exp $
//
// This is part of Realm Of Magic
//
// Usage: Class that handles the intermud <-> mud communication
//
// All rights reserved. See license.doc for complete information.
//
// Copyright (C) 1993-1996 by the Hackerz of the Realm Of Magic
// Realm Of Magic is based on CircleMUD, Copyright (C) 1993.
// CircleMUD is based on DikuMUD, Copyright (C) 1990, 1991.
// Original intermud_client.c (C) 1995, 96 by Chris Austin (Stryker@Tempus)
// Original lpc protocol (C) by Mark Lewis (Nostradamus@Zebedee)
//                           by J.W. Armstrong (Alvin@Sushi)
// ============================================================================

#include <string>
#include <stdio.h>
#include <iostream.h>
#include <iomanip.h>
#include <sys/types.h>
#include <unistd.h>
#include <sys/socket.h>
#include <sys/time.h>
#include <stdlib.h>

#include "mud.hh"
#include "log.hh"

extern _log interlog;
extern void close_sockets();

_mud::_mud()
{
  bullet.bytes = 0;  
}

void _mud::AcceptConnection()
{
  if ((file_descriptor = socket(AF_UNIX, SOCK_STREAM, 0)) == -1) {
    interlog << "ERROR: Can not open Unix Stream Socket." << logit;
    exit(1);
  }
  adress.sun_family = AF_UNIX;
  strcpy(adress.sun_path, MUDSOCK_PATH);
  unlink(adress.sun_path);
  adress_length = strlen(adress.sun_path) + sizeof(adress.sun_family);
  if (bind(file_descriptor, (struct sockaddr *) &adress,
      adress_length) == -1) {
    interlog << "SYSERR: Can not bind local mud socket! Check MUDSOCK_PATH "
                "in src/driver/include/intermud.hh" << logit;
    exit(1);
  }
  if (listen(file_descriptor, 1) == -1) {
    interlog << "SYSERR: Can not listen to file_descriptor!" << logit;
    exit(1);
  }
  interlog << "INFO: Waiting for connection to local mud!" << logit;
  file_descriptor = accept(file_descriptor, 0, 0);
  interlog << "INFO: Accepted connection from local mud!" << logit;
  lost_connection = false;
  return;
}

bool _mud::MudLost()
{
  return lost_connection;
}


void _mud::FlushData()
{
  int bytes;

  bytes = send_data();

  if (bytes <= 0) {
    interlog << "ERROR: Data sent to mud returned 0 bytes, exiting!" << logit;
    close_sockets();
    exit(1);
  }
  
  return;
}

string _mud::OutBuffer()
{
  return out_data;
}

int _mud::send_data() {

  int buflen, cc;
  char *buf;

  buf = strdup(out_data.c_str());

  if (strlen(buf) <= 0) {
    interlog << "ERROR: send_data requested to send 0 bytes, exiting" << logit;
    close_sockets();
    exit(1);
  }
  
  bullet.bytes = strlen(buf);

  cc = send(file_descriptor, (char *) &bullet, sizeof(struct bullet), 0);
  if (cc <= 0) {
    free(buf);
    return cc;
  } else {
    buflen = bullet.bytes;
    while (buflen > 0) {
      cc = send(file_descriptor, buf, buflen, 0);
      if (cc <= 0) {
        interlog << "ERROR: send_data, most probably fatal" << logit;
      }
      buf += cc;
      buflen -= cc;
    }
    return bullet.bytes;
  }
}

int _mud::recv_data(char *buf)
{
  int buflen, cc;

  cc = recv(file_descriptor, (char *) &bullet, sizeof(struct bullet), 0);
  if (cc <= 0)
    return cc;
  else {
    buflen = bullet.bytes;
    while (buflen > 0) {
      cc = recv(file_descriptor, buf, buflen, 0);
      if (cc <= 0) {
        interlog << "ERROR: recv_data, most probably fatal" << logit;
      }
      buf += cc;
      buflen -= cc;
    }
    return bullet.bytes;
  }
}

string _mud::ReceiveData()
{
  char raw_data[8192];
  int bytes;
  
  bytes = recv_data(raw_data);

  if (bytes <= 0) {
    interlog << "SYSERR: 0 bytes received on mud socket, "
                "mud has probably disappeared!" << logit;
    lost_connection = true;
    return "";
  } else
    raw_data[bytes] = '\0';

  in_data = raw_data;
  return in_data;
}

void _mud::CloseFD()
{
  close(file_descriptor);
  return;
}

_mud& _mud::operator<<(string s)
{
  this->out_data += s;
  return *this;
}

_mud& _mud::operator<<(int i)
{
  char num_buf[256];

  sprintf(num_buf, "%d", i);
  this->out_data += num_buf;
  return *this;
}

void _mud::AddToSet(fd_set &set)
{
  FD_SET(file_descriptor, &set);
  return;
}

bool _mud::CheckSet(fd_set &set)
{
  if (FD_ISSET(file_descriptor, &set))
    return true;
  
  return false;
}

void _mud::ClearSet(fd_set &set)
{
  FD_CLR(file_descriptor, &set);
  return;
}

int _mud::Desc()
{
  return file_descriptor;
}

_mud& _mud::operator<<(unsigned int ui)
{
  char num_buf[256];

  sprintf(num_buf, "%u", ui);
  this->out_data += num_buf;
  return *this;
}

_mud& _mud::operator<<(long int li)
{
  char num_buf[256];

  sprintf(num_buf, "%ld", li);
  this->out_data += num_buf;
  return *this;
}

_mud& _mud::operator<<(unsigned long int uli)
{
  char num_buf[256];
  
  sprintf(num_buf, "%lu", uli);
  this->out_data += num_buf;
  return *this;
}

_mud& _mud::operator<<(short int si)
{
  return (*this) << (int)si;
}

_mud& _mud::operator<<(unsigned short int usi)
{
  return (*this) << (unsigned int)usi;
}

_mud& _mud::operator<<(_mud& (*pf)(_mud&))
{
  return (*pf)(*this);
}

_mud& sendit(_mud& flush_mud)
{
  flush_mud.FlushData();
  flush_mud.out_data = "";
  return flush_mud;
}

