// ============================================================================
// $Id: utils.cc,v 1.1 1996/04/22 18:25:24 questor Exp $
//
// This is part of Realm Of Magic
//
// Usage: Utility functions for the Intermud server
//
// All rights reserved. See license.doc for complete information.
//
// Copyright (C) 1993-1996 by the Hackerz of the Realm Of Magic
// Realm Of Magic is based on CircleMUD, Copyright (C) 1993.
// CircleMUD is based on DikuMUD, Copyright (C) 1990, 1991.
// Original intermud_client.c (C) 1995, 96 by Chris Austin (Stryker@Tempus)
// Original lpc protocol (C) by Mark Lewis (Nostradamus@Zebedee)
//                           by J.W. Armstrong (Alvin@Sushi)
// ============================================================================

#include <time.h>
#include "utils.hh"
#include "log.hh"

extern _log interlog;

bool case_compare(string str1, string str2)
{
  int check, idx;

  if (str1.size() != str2.size())
    return true;

  for (idx = 0; idx < str1.size() || idx < str2.size(); idx++)
    if ((check = LOWER(str1[idx]) - LOWER(str2[idx])))
      return true;
  return false;

}

bool case_n_compare(string str1, string str2, int anz)
{
  int check, idx;

  for (idx = 0; (idx < str1.size() || idx < str2.size()) && idx < anz; idx++)
    if ((check = LOWER(str1[idx]) - LOWER(str2[idx])))
      return true;
  return false;
}

void update_timestamp(dns_record &dns)
{
  long int current_time;
  string stamp;

  current_time = time(0);
  stamp = asctime(localtime(&current_time));
  stamp[stamp.size() - 1] = '|';
  dns.last_connected = stamp;
  return;
}

void copy_dns_record(dns_record &copy, dns_record &org)
{
  copy.ip_address = org.ip_address;
  copy.port = org.port;
  copy.mud_name = org.mud_name;
  copy.mud_port = org.mud_port;
  copy.mud_version = org.mud_version;
  copy.intermud_version = org.intermud_version;
  copy.mud_type = org.mud_type;
  copy.kb_in = org.kb_in;
  copy.kb_out = org.kb_out;
  copy.msg_in = org.msg_in;
  copy.msg_out = org.msg_out;
  copy.muted = org.muted;
  copy.services = org.services;
  copy.last_connected = org.last_connected;
  return;
}

string encode_services(struct dns_record &remote_mud)
{
  int   i;
  string services;
  string tmpbuf;
  string tmpsubbuf;

  tmpbuf = "00000000000000000000000000000000";
  services = "00000000";

  i = 0;
  if (IS_SET(remote_mud.services, ST_INTERWIZ))
    tmpbuf[i] = '1';

  i++;

  if (IS_SET(remote_mud.services, ST_INTERTELL))
    tmpbuf[i] = '1';

  i++;

  if (IS_SET(remote_mud.services, ST_INTERPAGE))
    tmpbuf[i] = '1';

  i++;

  if (IS_SET(remote_mud.services, ST_INTERWHO))
    tmpbuf[i] = '1';

  i++;

  for (i = 0;i <= 7;i++) {
    tmpsubbuf = tmpbuf.substr(i * 4, 4);
    if (tmpsubbuf == "1111")
      services[i] = 'F';
    else if (tmpsubbuf == "1110")
      services[i] = 'E';
    else if (tmpsubbuf == "1101")
      services[i] = 'D';
    else if (tmpsubbuf == "1100")
      services[i] = 'C';
    else if (tmpsubbuf == "1011")
      services[i] = 'B';
    else if (tmpsubbuf == "1010")
      services[i] = 'A';
    else if (tmpsubbuf == "1001")
      services[i] = '9';
    else if (tmpsubbuf == "1000")
      services[i] = '8';
    else if (tmpsubbuf == "0111")
      services[i] = '7';
    else if (tmpsubbuf == "0110")
      services[i] = '6';
    else if (tmpsubbuf == "0101")
      services[i] = '5';
    else if (tmpsubbuf == "0100")
      services[i] = '4';
    else if (tmpsubbuf == "0011")
      services[i] = '3';
    else if (tmpsubbuf == "0010")
      services[i] = '2';
    else if (tmpsubbuf == "0001")
      services[i] = '1';
    else if (tmpsubbuf == "0000")
      services[i] = '0';
  }

  return services;
}

void decode_services(string bitmap, struct dns_record &remote_mud)
{
  int i = 0, j = 0;
  string tmpbuf;

  while (j < 8) {
    switch (bitmap[j]) {
      case 'F' : tmpbuf += "1111"; break;
      case 'E' : tmpbuf += "1110"; break;
      case 'D' : tmpbuf += "1101"; break;
      case 'C' : tmpbuf += "1100"; break;
      case 'B' : tmpbuf += "1011"; break;
      case 'A' : tmpbuf += "1010"; break;
      case '9' : tmpbuf += "1001"; break;
      case '8' : tmpbuf += "1000"; break;
      case '7' : tmpbuf += "0111"; break;
      case '6' : tmpbuf += "0110"; break;
      case '5' : tmpbuf += "0101"; break;
      case '4' : tmpbuf += "0100"; break;
      case '3' : tmpbuf += "0011"; break;
      case '2' : tmpbuf += "0010"; break;
      case '1' : tmpbuf += "0001"; break;
      case '0' : tmpbuf += "0000"; break;
      default  : interlog << "SYSERR: Invalid bitmap from "
                          << remote_mud.mud_name << logit;
                 exit(1);
    }

    j++;
  }
  for (i = 0; i < 5; i++) {
    if (i == 0 && tmpbuf[i] == '1')
      SET_BIT(remote_mud.services, ST_INTERWIZ);
    else if (i == 1 && tmpbuf[i] == '1')
      SET_BIT(remote_mud.services, ST_INTERTELL);
    else if (i == 2 && tmpbuf[i] == '1')
      SET_BIT(remote_mud.services, ST_INTERPAGE);
    else if (i == 3 && tmpbuf[i] == '1')
      SET_BIT(remote_mud.services, ST_INTERWHO);
  }
  return;
}

